(* Title: Specify/p-model.sml
   Author: Walther Neuper 170207
   (c) due to copyright terms

This will be dropped at switch to Isabelle/PIDE .
*)

signature PRESENTATION_MODEL =
sig
  datatype switch_pbl_met = Model_For_Problem_Ref | Model_For_Method_Ref

  type model_out
  val template_args: switch_pbl_met -> References.input -> Ctree.ctree ->
    (switch_pbl_met * model_out) * References.input

  (* for the OLD P_Model.T *)
  type T
  val empty: T
  val to_string: T -> string

  datatype item = (*shall be dropped with PIDE*)
    Correct of TermC.as_string | False of TermC.as_string | Incompl of TermC.as_string
  | Missing of TermC.as_string | Superfl of string | SyntaxE of string | TypeE of string
  type 'a model     (*shall be dropped with PIDE*)

  val from : theory -> I_Model.T -> Pre_Conds.T -> T

  val to_list: 'a model -> 'a list
  val mk_delete: theory -> string -> I_Model.feedback -> Tactic.input
  val mk_additem: string -> TermC.as_string -> Tactic.input

  val item_from_feedback: theory -> I_Model.feedback -> item

\<^isac_test>\<open>
(**)
\<close>
end

(**)
structure P_Model(**) : PRESENTATION_MODEL(**) =
struct
(**)

datatype switch_pbl_met = Model_For_Problem_Ref | Model_For_Method_Ref


(** for the NEW template for Specification**)
datatype feedback = Cor | Inc | Sup | True | False
(*
  legend:
  Cor   correct: the default representation
  Inc   incomplete list, (accessible) markup to be defined
  Sup   superfluous within the current variant from Formalise.model;
        (accessible) markup to be defined
  True  the default representation
  False (accessible) markup to be defined
*)
type out_element = UnparseC.term_as_string * feedback
type field_out = Model_Pattern.m_field * out_element list
type model_out =
    field_out (* "#Given", feedback: Cor | Inc | Sup *)
  * field_out (* "#Where", feedback: True | False    *)
  * field_out (* "#Find",  feedback: Cor | Inc | Sup *)
  * field_out (* "#Relate",feedback: Cor | Inc | Sup *)

(*val item_to_out: Proof.context -> I_Model.single -> out_element*)
fun item_to_out ctxt (_, _, _, _, I_Model.Cor (dsc_ts, _)) =
    (dsc_ts |> Term.list_comb |> UnparseC.term_in_ctxt ctxt, Cor)
  | item_to_out ctxt (_, _, _, _, I_Model.Inc (dsc_ts, _)) =
    (dsc_ts |> Term.list_comb |> UnparseC.term_in_ctxt ctxt, Inc)
  | item_to_out ctxt ((_, _, _, _, I_Model.Sup dsc_ts): I_Model.single) =
    (dsc_ts |> Term.list_comb |> UnparseC.term_in_ctxt ctxt, Sup)
  | item_to_out ctxt im =
    raise ERROR ("P_Model.item_to_out called with " ^ I_Model.single_to_string ctxt im)

(*val items_to_out: Proof.context -> Model_Pattern.m_field -> I_Model.T -> field_out*)
fun items_to_out ctxt m_field items =
  (m_field,
    filter (fn (_, _, _, f, _) => f = m_field) items |> map (item_to_out ctxt))

(*val where_to_out: Proof.context -> bool * term -> out_element*)
fun where_to_out ctxt (bool, t) =
  (UnparseC.term_in_ctxt ctxt t, if bool then True else False)

(*val wheres_to_out: Proof.context -> (bool * term) list -> field_out*)
fun wheres_to_out ctxt wheres = ("#Where", map (where_to_out ctxt) wheres)

(*val model_to_out: Proof.context -> (bool * term) list -> I_Model.T -> field_out list*)
(* the sequence of the template is maitained *)
fun model_to_out ctxt where_ model =
  let
    val given = items_to_out ctxt "#Given" model
    val where_ = wheres_to_out ctxt where_
    val find = items_to_out ctxt "#Find" model
    val relate = items_to_out ctxt "#Relate" model
  in (given, where_, find, relate) end

fun template_args switch_pbl_met input_refs store =
  if not (Ctree.is_pblnd store) then raise ERROR "update code w.r.t. to future design of Solution"
  else
    let
      val Ctree.Nd (Ctree.PblObj {origin = (_, o_refs, _), probl, meth, spec, ctxt, ...}, []) = store
      val (_, pbl_id, met_id) = References.select_input o_refs spec
      val p_model = 
        case switch_pbl_met of
          Model_For_Problem_Ref =>
            let (* the sequence *)
              val {where_rls, where_, ...} = Problem.from_store ctxt pbl_id
              val (_, where_) = Pre_Conds.check ctxt where_rls where_ probl 0
            in model_to_out ctxt where_ probl end
        | Model_For_Method_Ref =>
            let
              val {where_rls, where_ = where_, ...} = MethodC.from_store ctxt met_id
              val (_, where_) = Pre_Conds.check ctxt where_rls where_ meth 0
            in model_to_out ctxt where_ meth end
    in
      ((switch_pbl_met, p_model), input_refs)
    end

(** types for OLD presentation**)

type 'a model = 
  {Given : 'a list, Where: 'a list, Find  : 'a list, With : 'a list, Relate: 'a list};
datatype item = 
    Correct of TermC.as_string (* labels a correct formula (type cterm') *)
  | SyntaxE of string (**)
  | TypeE   of string (**)
  | False   of TermC.as_string (* WN050618 notexistent in itm_: only used in Where *)
  | Incompl of TermC.as_string (**)
  | Superfl of string (**)
  | Missing of TermC.as_string; (*REVIEW: a "missing" item is not shown, thus cannot be marked*)

type T = item model;
fun item2str (Correct  s) = "Correct " ^ s
  | item2str (SyntaxE  s) = "SyntaxE " ^ s
  | item2str (TypeE    s) = "TypeE " ^ s
  | item2str (False    s) = "False " ^ s
  | item2str (Incompl  s) = "Incompl " ^ s
  | item2str (Superfl  s) = "Superfl " ^ s
  | item2str (Missing  s) = "Missing " ^ s;

fun to_string ({Given=Given,Where=Where,
		 Find=Find,With=With,Relate=Relate}:item model)=
    ("{Given =" ^ ((strs2str' o (map item2str))	 Given ) ^
     ",Where=" ^ ((strs2str' o (map item2str))	 Where) ^
     ",Find  =" ^ ((strs2str' o (map item2str))	 Find  ) ^
     ",With =" ^ ((strs2str' o (map item2str))	 With ) ^
     ",Relate=" ^ ((strs2str' o (map item2str))	 Relate) ^ "}");

val empty = {Given = [], Where= [], Find  = [], With = [], Relate= []};


(** build T **)

fun item_from_feedback thy (I_Model.Cor ((d, ts), _)) = Correct (UnparseC.term_in_thy thy (Input_Descript.join (d, ts)))
  | item_from_feedback _ (I_Model.Syn c) = SyntaxE c
  | item_from_feedback _ (I_Model.Typ c) = TypeE c
  | item_from_feedback thy (I_Model.Inc ((d, ts), _)) = Incompl (UnparseC.term_in_thy thy  (Input_Descript.join (d, ts)))
  | item_from_feedback thy (I_Model.Sup (d, ts)) = Superfl (UnparseC.term_in_thy thy  (Input_Descript.join (d, ts)))
  | item_from_feedback thy (I_Model.Mis (d, pid)) = Missing (UnparseC.term_in_thy thy d ^ " " ^ UnparseC.term_in_thy thy  pid)
  | item_from_feedback _ _ = raise ERROR "item_from_feedback: uncovered definition"

fun add_sel_ppc (_: theory) sel {Given = gi, Where = wh, Find = fi, With = wi, Relate = re} x =
  case sel of
    "#Given" => {Given = gi @ [x], Where = wh, Find = fi, With = wi, Relate = re}
  | "#Where" => {Given = gi, Where = wh @ [x], Find = fi, With = wi, Relate = re}
  | "#Find"  => {Given = gi, Where = wh, Find = fi @ [x], With = wi, Relate = re}
  | "#Relate"=> {Given = gi, Where = wh, Find = fi, With = wi, Relate = re @ [x]}
  | "#undef" => {Given = gi @ [x], Where = wh, Find = fi, With = wi, Relate = re} (*ori2itmSup*)
  | _  => raise ERROR ("add_sel_ppc tried to select by \"" ^ sel ^ "\"")
fun add_where {Given = gi, Where = _, Find = fi, With = wi, Relate = re} wh =
  {Given = gi, Where = wh, Find= fi , With = wi, Relate = re}

fun boolterm2item ctxt (true, term) = Correct (UnparseC.term_in_ctxt ctxt term)
  | boolterm2item ctxt(false, term) = False (UnparseC.term_in_ctxt ctxt term);

fun from thy itms where_ =
  let
    fun coll model [] = model
      | coll model ((_, _, _, field, itm_) :: itms) =
        coll (add_sel_ppc thy field model (item_from_feedback thy itm_)) itms;
    val gfr = coll empty itms;
  in add_where gfr (map (boolterm2item (Proof_Context.init_global thy)) where_) end;

fun mk_delete thy "#Given" itm_ = Tactic.Del_Given (I_Model.to_p_model thy itm_)
  | mk_delete thy "#Find" itm_ = Tactic.Del_Find (I_Model.to_p_model thy itm_)
  | mk_delete thy "#Relate" itm_ = Tactic.Del_Relation (I_Model.to_p_model thy itm_)
  | mk_delete _ str _ = raise ERROR ("mk_delete: called with field \"" ^ str ^ "\"")
fun mk_additem "#Given" ct = Tactic.Add_Given ct
  | mk_additem "#Find" ct = Tactic.Add_Find ct    
  | mk_additem "#Relate"ct = Tactic.Add_Relation ct
  | mk_additem str _ = raise ERROR ("mk_additem: called with field \"" ^ str ^ "\"")

fun to_list {Given = gis, Where = whs, Find = fis, With = wis, Relate = res} =
  gis @ whs @ fis @ wis @ res

(**)end(**);