signature SUB_PROBLEM =
sig
  val tac_from_prog: Ctree.state -> term -> Tactic.input * Tactic.T
  val common_sub_thy: theory * theory -> theory
end

(**)
structure Sub_Problem(**): SUB_PROBLEM(**) =
struct
(**)

fun common_sub_thy (thy1, thy2) =
  if Context.subthy (thy1, thy2) then thy2
  else if Context.subthy (thy2, thy1) then thy1
    else Know_Store.get_ref_last_thy ()

fun tac_from_prog (pt, p) (stac as Const (\<^const_name>\<open>Prog_Tac.SubProblem\<close>, _) $ spec' $ ags') =
    let
      val (dI, pI, mI) = Prog_Tac.dest_spec spec'
      val thy = common_sub_thy (Know_Store.get_via_last_thy dI, Ctree.root_thy pt);
      val init_ctxt = Proof_Context.init_global thy
	    val ags = TermC.isalist2list ags';
	    val (pI, pors, mI) = 
	      if mI = ["no_met"]
	      then
          let
            val pors = (M_Match.arguments thy ((#model o (Problem.from_store init_ctxt)) pI) ags): O_Model.T
		          handle ERROR "actual args do not match formal args"
			          => (M_Match.arguments_msg init_ctxt pI stac ags (*raise exn*);[]);
		        val pI' = Refine.refine_ori' init_ctxt pors pI;
		      in (pI', pors (* refinement over models with diff.precond only *), 
		          (hd o #solve_mets o Problem.from_store init_ctxt) pI') end
	      else (pI, (M_Match.arguments thy ((#model o Problem.from_store init_ctxt) pI) ags)
		      handle ERROR "actual args do not match formal args"
		      => (M_Match.arguments_msg init_ctxt pI stac ags(*raise exn*); []), mI);
      val (fmz_, vals) = O_Model.values' init_ctxt pors;
	    val {cas, model, thy, ...} = Problem.from_store init_ctxt pI
	    val dI = Context.theory_name thy (*take dI from _refined_ pbl*)
	    val dI = Context.theory_name
        (common_sub_thy (Know_Store.get_via_last_thy dI, Ctree.root_thy pt))
	    val ctxt = ContextC.initialise init_ctxt vals
	    val hdl =
        case cas of
		      NONE => Auto_Prog.pblterm dI pI
		    | SOME t => subst_atomic ((Model_Pattern.variables model) ~~~ vals) t
      val f = Auto_Prog.subpbl (strip_thy dI) pI
    in
      (Tactic.Subproblem (dI, pI),	Tactic.Subproblem' ((dI, pI, mI), pors, hdl, fmz_, ctxt, f))
    end
  | tac_from_prog _ t = raise ERROR ("Sub_Problem.tac_from_prog not impl. for " ^ UnparseC.term t)

(**)end(**)
